--
-- Mutilate -- spell recommendation for rogues. Originlly derived from FaceMauler.
-- 

Mutilate = {} -- Our addon-specific 'global' non-saved variables.
local ns = Mutilate -- A file private shorter namespace to reference them by.
local _ -- So we don't pollute the global _
local options -- Ref to our saved options. Will be set during load.

ns.addOnName = "Mutilate" -- To make cloning easier.

ns.lastUpdateTime = 0


-- Return a localised spell name.
function ns.GetSpellNameById(spellId)
	local spellName, _, _, _, _, _, _, _, _ = GetSpellInfo(spellId)
	return spellName
end

-- Energy costs.
ns.e = {}
ns.e.Vendetta = 0
ns.e.Garrote = 45
ns.e.Mutilate = 55
ns.e.Envenom  = 35
ns.e.Rupture = 25
ns.e.Tricks_of_the_Trade = 0
ns.e.Vanish = 0
ns.e.Hemorrhage = 30
	
local language = GetLocale();

-- Local names of spells we are interested in, indexed by english name. We need to use the
-- local name (or id) in calls refering to the spell. We chose to use local name rather
-- than id because debugging is easier.
ns.n = {}
ns.n.Vendetta            = ns.GetSpellNameById(79140)
ns.n.Garrote             = ns.GetSpellNameById(703)
ns.n.Mutilate            = ns.GetSpellNameById(1329)
ns.n.Envenom             = ns.GetSpellNameById(32645)
ns.n.Rupture             = ns.GetSpellNameById(1943)
ns.n.Tricks_of_the_Trade = ns.GetSpellNameById(57934)
ns.n.Vanish              = ns.GetSpellNameById(1856)
ns.n.Kick           	 = ns.GetSpellNameById(1766)
ns.n.Shiv 				 = ns.GetSpellNameById(5938)
ns.n.Cloak_of_Shadows 	 = ns.GetSpellNameById(31224)
ns.n.Hemorrhage 		 = ns.GetSpellNameById(16511) -- Speed +25, not like h E1 m O r @ dZ
ns.n.Exsanguinate		 = ns.GetSpellNameById(200806) -- E k s {1 n g u n EI 4

ns.id = {}
ns.id.Assasination = 259
ns.id.Outlaw = 260
ns.id.Subtlety = 261

-- Our sneaky frame to watch for events ... checks na.events[] for the function.  Passes all args.
ns.eventFrame = CreateFrame("Frame")
ns.eventFrame:SetScript("OnEvent", function(this, event, ...) ns.events[event](...) end)

ns.eventFrame:RegisterEvent("ADDON_LOADED")
ns.eventFrame:RegisterEvent("PLAYER_ALIVE")
ns.eventFrame:RegisterEvent("PLAYER_TARGET_CHANGED")
ns.eventFrame:RegisterEvent("PLAYER_TALENT_UPDATE")
ns.eventFrame:RegisterEvent("SPELLS_CHANGED")
--ns.eventFrame:RegisterEvent("UNIT_AURA")

-- Define our Event Handlers here
ns.events = {}

function ns.CheckSpec()
	local currentSpec = GetSpecialization()
	local currentSpecID = currentSpec and select(1, GetSpecializationInfo(currentSpec)) or 0
	if currentSpecID == ns.id.Assasination then -- 259 is Assasination spec
		ns.spec = "Assasination"
		ns.displayFrame:Show() 
	elseif currentSpecID == 102 then
		ns.spec = "Balance"
		ns.displayFrame:Show() 
	else
		ns.displayFrame:Hide() 
	end 	
end

function ns.events.PLAYER_TALENT_UPDATE()
	ns.CheckSpec()

	-- GetTalentInfo(tier, column, talentGroup)

	-- Talent group 1: Assasination
	local talentId, name, texture, selected, available = GetTalentInfo(1, 3, 1)
	ns.hasHemorrhageTalent = selected
	local talentId, name, texture, selected, available = GetTalentInfo(6, 3, 1)
	ns.hasExsanguinateTalent = selected
	local talentId, name, texture, selected, available = GetTalentInfo(2, 3, 1)
	ns.hasShadowFocusTalent = selected
	local talentId, name, texture, selected, available = GetTalentInfo(3 ,2, 1)
	ns.hasAnticipation = selected
	local inactive_regen_per_sec, active_regen_per_sec = GetPowerRegen()
	ns.combatEnergyRegenPerSec = active_regen_per_sec
end

function ns.events.PLAYER_ALIVE()

	ns.eventFrame:UnregisterEvent("PLAYER_ALIVE")
end

function ns.events.ADDON_LOADED(addon)

	--for k, v in pairs(ns.L) do print(v, k) end

	if addon ~= ns.addOnName then 
		return 
	end

	local _,playerClass = UnitClass("player")
	if playerClass ~= "ROGUE" then  --and playerClass ~= "DRUID" then
		ns.eventFrame:UnregisterEvent("PLAYER_ALIVE")
		ns.eventFrame:UnregisterEvent("ADDON_LOADED")
		ns.eventFrame:UnregisterEvent("PLAYER_LOGIN")
		ns.eventFrame:UnregisterEvent("PLAYER_TARGET_CHANGED")
		ns.eventFrame:UnregisterEvent("PLAYER_TALENT_UPDATE")
		return 
	end
	
	-- Default saved variables
	if not Mutilatedb then
		Mutilatedb = {} -- fresh start
	end
	options = Mutilatedb -- Generic file-local short name.

	if not options.scale then options.scale = 1 end
	if options.locked == nil then options.locked = false end
	if not options.x then options.x = 100 end
	if not options.y then options.y = 100 end
	if not options.SuggestVanish then options.SuggestVanish = false end
	if not options.SuggestVendetta then options.SuggestVendetta = false end
	if not options.SliceAndDiceTimer then options.SliceAndDiceTimer = 1.0 end
	if not options.RuptureTimer then options.RuptureTimer = 2.0 end
	if not options.RuptureClipTimer then options.RuptureClipTimer = 0.0 end
	if not options.PoolEnergyThreshold then options.PoolEnergyThreshold = 90 end
	if not options.RuptureCp then options.RuptureCp = 1 end
	if not options.EnvenomCp then options.EnvenomCp = 4 end
	if not options.PlayerHealthThreshold then options.PlayerHealthThreshold = 50 end
	if not options.EnvenomTimer then options.EnvenomTimer = 0 end
	if not options.BackstabEnvenomCp then options.BackstabEnvenomCp = 5 end
	if not options.SuggestMutAsStealthOpener then options.SuggestMutAsStealthOpener = true end
	if options.SuggestCloakOfShadows == nil then options.SuggestCloakOfShadows = true end
	if not options.MinDebuffDurationForCofs then options.MinDebuffDurationForCofs = 10 end
	if options.VoicePrompts == nil then options.VoicePrompts = true end


	-- Create GUI
	ns.CreateGUI()
	ns.displayFrame:SetScale(options.scale)


	-- Create Options Frame
	ns.CreateOptionFrame()
	if options.locked then
		ns.displayFrame:SetScript("OnMouseDown", nil)
		ns.displayFrame:SetScript("OnMouseUp", nil)
		ns.displayFrame:SetScript("OnDragStop", nil)
		ns.displayFrame:SetBackdropColor(0, 0, 0, 0)
		ns.displayFrame:EnableMouse(false)
	else
		ns.displayFrame:SetScript("OnMouseDown", function(self) self:StartMoving() end)
		ns.displayFrame:SetScript("OnMouseUp", function(self) self:StopMovingOrSizing() end)
		ns.displayFrame:SetScript("OnDragStop", function(self) self:StopMovingOrSizing() end)
		ns.displayFrame:SetBackdropColor(0, 0, 0, .4)
		ns.displayFrame:EnableMouse(true)
	end

	-- Register for Slash Commands
	SlashCmdList["Mutilate"] = ns.Options -- Defined below.
	SLASH_Mutilate1 = "/Mutilate"
	SLASH_Mutilate2 = "/mut"

	-- Register for Function Events
	ns.eventFrame:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	ns.eventFrame:RegisterEvent("PLAYER_TARGET_CHANGED")

	--ns.events.PLAYER_TALENT_UPDATE()

	ns.CheckSpec()

	-- This is probably normally caled by framework, but just be be sure. Applies spec and talent based settings.
	ns.events.PLAYER_TALENT_UPDATE()
	
end

-- Show or hide the spell textures.
function ns.ShowSpellsIf(yes)
	if not yes then
		ns.displayFrameRecommend:Hide()
		ns.displayFrameC1:Hide()
		ns.displayFrameA1:Hide()
		ns.displayFrameB1:Hide()
		ns.displayFrameB2:Hide()
		ns.displayFrameA2:Hide()
		ns.displayFrameD1:Hide()
		ns.bottomText:SetText("")
		ns.rightSideText:SetText("")
		ns.saidSpell = nil
		ns.saidKick = false
	else
		ns.displayFrameRecommend:Show()
		ns.displayFrameC1:Show()
		ns.displayFrameA1:Show()
		ns.displayFrameB1:Show()
		ns.displayFrameB2:Show()
		ns.displayFrameA2:Show()
		ns.displayFrameD1:Show()
		ns.DecideSpells()
	end
end


function ns.events.COMBAT_LOG_EVENT_UNFILTERED(timestamp, event, hideCaster, srcGUID, srcName, srcFlags, srcRaidFlags, dstGUID, dstName, dstFlags, destRaidFlags, ...)

	ns.ShowSpellsIf(UnitName("target") ~= nil and not UnitIsFriend("player", "target") and UnitHealth("target") > 0)

end

function ns.events.PLAYER_TARGET_CHANGED(...)
	
	ns.ShowSpellsIf(UnitName("target") ~= nil and not UnitIsFriend("player", "target") and UnitHealth("target") > 0)

end

function ns.events.SPELLS_CHANGED(...)

	if not ns.CheckSpec() then
		return
	end

	-- TODO
end

-- End Event Handlers

function ns.CreateFrameAndTextureOnGrid(displayFrameName, textureName, tlx, tly, width, height)
  local df = CreateFrame("Frame", "$parent_"..displayFrameName, ns.displayFrame)
  ns[displayFrameName] = df
  df:SetWidth(width)
  df:SetHeight(height)
  df:SetPoint("TOPLEFT", tlx, tly)
  local t = df:CreateTexture(nil, "BACKGROUND")
  ns[textureName] = t
  t:SetTexture(nil)
  t:SetAllPoints(df)
  t:SetAlpha(.8)
  df.texture = t -- Necessary?
  return t
end

function ns.CreateGUI()

	local gdf = CreateFrame("Frame", "MutilateFrame", UIParent)
	ns.displayFrame = gdf
	gdf:SetFrameStrata("BACKGROUND")
	gdf:SetWidth(350)
	gdf:SetHeight(107)
	gdf:SetBackdrop({bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", tile = true, tileSize = 32})
	gdf:SetBackdropColor(0, 0, 0, .4)
	gdf:EnableMouse(true)
	gdf:SetMovable(true)
	gdf:SetClampedToScreen(true)
	gdf:SetScript("OnMouseDown", function(self) self:StartMoving() end)
	gdf:SetScript("OnMouseUp", function(self) self:StopMovingOrSizing() end)
	gdf:SetScript("OnDragStop", function(self) self:StopMovingOrSizing() end)
	gdf:SetPoint("CENTER", -200, -200)

    -- Frames are named like a spreadsheet around the central recommenndation. So C1 in the next spell.
    -- A1 B1 REC C1 D1 __right side text under D1
    -- A2 B2 OMM 
    -- bottom text...
	ns.CreateFrameAndTextureOnGrid("displayFrameRecommend", "textureRecommend", 140, -10, 70, 70)
	ns.CreateFrameAndTextureOnGrid("displayFrameA1", "textureA1", 0, 0, 45, 45)
	ns.CreateFrameAndTextureOnGrid("displayFrameA2", "textureA2", 0, -50, 45, 45)
	ns.CreateFrameAndTextureOnGrid("displayFrameB1", "textureB1", 50, 0, 45, 45)
	ns.CreateFrameAndTextureOnGrid("displayFrameB2", "textureB2", 50, -50, 45, 45)
	ns.CreateFrameAndTextureOnGrid("displayFrameC1", "textureC1", 250, 0, 45, 45)
	ns.CreateFrameAndTextureOnGrid("displayFrameD1", "textureD1", 300, 0, 45, 45)

	ns.bottomText = gdf:CreateFontString("kickName", "OVERLAY", "GameFontNormal")
	ns.bottomText:SetText("")
	ns.bottomText:SetPoint("TOPLEFT", 0, -95)
	ns.rightSideText = gdf:CreateFontString("cofsName", "OVERLAY", "GameFontNormal")
	ns.rightSideText:SetJustifyH("RIGHT")
	ns.rightSideText:SetText("")
	ns.rightSideText:SetPoint("TOPRIGHT", 0, -45)

	gdf:SetScript("OnUpdate", function(this, elapsed) ns.OnUpdate(elapsed) end)
end

function ns.OnUpdate(elapsed)
		ns.DecideSpells()
end

-- Return when this spell will expire. 0 or some value now or less if it has already.
function ns.getExpirationTime(spellName)
	local name, _, _, _, _, _, expirationTime, isMine, _ = UnitDebuff("target", spellName)
	if name ~= nil and isMine == "player" then
		return expirationTime
	end
	return 0
end

-- Return when this spell will be available for casting with respect to its cooldown, and the global
-- cooldown. Returns 0 if possible for easier debug.
function ns.getReadyTime(spellName, s)
	local start, duration, enabled = GetSpellCooldown(spellName);
	if duration ~= nil and start + duration > s.now + 0.01 then 
		return max(start + duration, s.globalCoolDownExpiration)
	end
	if s.globalCoolDownExpiration > s.now + 0.01 then
		return s.globalCoolDownExpiration
	end
	return 0
end



-- Return a table with all the interesting facts about the current state. All times are absolute.
-- Ready times take into account global cooldown. Buffs and debuffs are active if their expiration
-- is in the future. Assumes target existis and is unfriendly.
-- Returns nil if we are in first few hundred milliseconds of the start of global cooldown, in
-- which state is often confused. In this situation, caller should assume a spell has been cast
-- (i.e. clear the recommendation) and hold predition as is.
function ns.GetState(now)

	local globalCoolDownExpiration = now
	local start, duration, enabled = GetSpellCooldown(61304); -- The speical global cooldown spell ID.
	if duration ~= nil and duration ~= 0 then
		globalCoolDownExpiration = max(now, start + duration)
		if now - start < 0.3 then 
			return nil
		end
	end

	local s = {}
	s.now = now
	s.globalCoolDownExpiration = globalCoolDownExpiration

	s.energy = UnitPower("player")
	s.comboPoints = GetComboPoints("player", "target")
	s.isInGroup = IsInGroup()

	s.removeableName = ""
	s.longDurationDebuffs = false
	-- Scan player's debuffs for things Cloak of Shadows could remove.
	for i = 1, 40 do -- There are 1..40 aura slots.
		local name, _, _, count, debuffType, _, expirationTime, unitCaster, canStealOrPurge, _, _, _, _ = UnitDebuff("player", i)
		if name == nil or unitCaster == "player" then
			break 
		end
		if expirationTime > s.now then
			local duration = math.floor(expirationTime - s.now)
			if duration < 60 * 10 then -- Don't show times > 10m
				s.removeableName = s.removeableName..""..duration.."s "
			end
			if duration > options.MinDebuffDurationForCofs then
				s.longDurationDebuffs = true
			end
		else
			s.longDurationDebuffs = true
		end
		if count > 1 then
			s.removeableName = s.removeableName..count.."*"
		end
		s.removeableName = s.removeableName..name.."\n"
	end

	-- See if there is something we can kick.
	s.interruptibleName = nil
	local name, _, displayName, _, _, endTime, _, notInterruptible = UnitChannelInfo("target")
	if name ~= nil and not notInterruptible then
		s.interruptibleName = displayName
	end
	local name, _, displayName, _, _, endTime, _, _, notInterruptible = UnitCastingInfo("target")
	if name ~= nil and not notInterruptible then
		s.interruptibleName = displayName
	end

	s.garroteExpiration = ns.getExpirationTime(ns.n.Garrote)
	s.hemorrhageExpiration = ns.getExpirationTime(ns.n.Hemorrhage)
	s.ruptureExpiration = ns.getExpirationTime(ns.n.Rupture)
	s.envenomExpiration = ns.getExpirationTime(ns.n.Envenom)

	s.vendettaReadyTime = ns.getReadyTime(ns.n.Vendetta, s)
	s.tricksOfTheTradeReadyTime = ns.getReadyTime(ns.n.Tricks_of_the_Trade, s)
	s.vanishReadyTime = ns.getReadyTime(ns.n.Vanish, s)
	s.kickReadyTime = ns.getReadyTime(ns.n.Kick, s)
	s.cofsReadyTime = ns.getReadyTime(ns.n.Cloak_of_Shadows, s)
	s.garroteReadyTime = ns.getReadyTime(ns.n.Garrote, s)
	s.exsanguinateReadyTime = ns.getReadyTime(ns.n.Exsanguinate, s)

	s.playerIsStealthed = UnitAura("player", "Stealth") ~= nil

	s.targetHealthFraction = UnitHealth("target") / UnitHealthMax("target")

	s.playerHealthFraction = UnitHealth("player") / UnitHealthMax("player")

	--for k, v in pairs(s) do print(v, k) end
	return s
end

function ns.StateSummary(s)
	local summary = ""
	for k, v in pairs(s) do
		if v ~= 0 and v ~= false and v ~= "" then
			summary = summary .. " " .. k .. "=" .. tostring(v)
		end
	end
	return summary
end

-- Determine a recommened spell, if any, from the given state. In the process, advance the state to
-- some future time where something might be different. Always advances the time of the state by at
-- least a little.
-- Returned spell is not localised.
function ns.GetSpellFromState(s, debugStr)
	-- http://www.icy-veins.com/wow/assassination-rogue-pve-dps-rotation-cooldowns-abilities
	-- As of 2016-07-24

	local spell = nil;
	local why = "no case";
	local oldNow = s.now
	local fullCombos = 4
	if (ns.hasAnticipation) then
		fullCombos = 7 -- Check this.
	end

	local eCostMult = 1
	if s.playerIsStealthed and ns.hasShadowFocusTalent then
		eCostMult = 0.5
	end

	if s.globalCoolDownExpiration > s.now then

		why = "waiting for global cooldown"
		s.now = s.globalCoolDownExpiration 

	elseif ns.hasExsanguinateTalent
		and s.exsanguinateReadyTime < s.now
		and s.ruptureExpiration > s.now + 8 -- say
		and s.garroteExpiration > s.now + 6 -- say
		then

		spell = "Exsanguinate"
		why = "bleeds up, can Exsanguinate"
		s.ruptureExpiration = s.now + (s.ruptureExpiration - s.now) / 4
		s.garroteExpiration = s.now + (s.garroteExpiration - s.now) / 4
		s.exsanguinateReadyTime = s.now + 45

	elseif s.energy >= ns.e.Rupture * eCostMult
		and ((s.ruptureExpiration < s.now + 8 and s.comboPoints >= fullCombos)
			or (s.ruptureExpiration < s.now + 2 and s.comboPoints >= 1)) then

		-- "It is optimal to apply and refresh, with 8 seconds or less remaining, Rupture with 5 Combo Points.
		-- That said, if it is about to expire, refresh it with whatever amount of Combo Points you have."
		spell = "Rupture"
		why = "it needs refreshing"
		s.ruptureExpiration = s.now + s.comboPoints * 5 -- Approx
		s.comboPoints = 0
		s.energy = s.energy - ns.e.Rupture * eCostMult
		s.playerIsStealthed = false

	elseif s.energy >= ns.e.Envenom * eCostMult
		and s.energy >= UnitPowerMax("player") * 0.95
		and s.comboPoints > 0 then

		spell = "Envenom"
		why = "it is better than wasting energy"
		s.comboPoints = 0
		s.energy = s.energy - ns.e.Envenom * eCostMult
		s.envenomExpiration = s.now + s.comboPoints * 2 -- Approx
		s.playerIsStealthed = false

	elseif s.energy >= ns.e.Envenom * eCostMult and s.comboPoints >= fullCombos then

		spell = "Envenom"
		why = "it needs refreshing"
		s.comboPoints = 0
		s.energy = s.energy - ns.e.Envenom * eCostMult
		s.envenomExpiration = s.now + s.comboPoints * 2 -- Approx
		s.playerIsStealthed = false

	elseif s.comboPoints < 5 then

		-- Combo point building...
		if s.energy >= ns.e.Garrote * eCostMult
			and s.garroteExpiration < s.now + 2
			and s.garroteReadyTime <= s.now then

			spell = "Garrote"
			why = "no garrote bleed active and you need combo points"
			s.comboPoints = s.comboPoints + 1
			s.energy = s.energy - ns.e.Garrote * eCostMult
			s.garroteExpiration = s.now + 18
			s.garroteReadyTime = s.now + 15
			s.playerIsStealthed = false

		elseif ns.hasHemorrhageTalent
			and s.energy >= ns.e.Hemorrhage * eCostMult
			and s.hemorrhageExpiration < s.now + 2 then

			spell = "Hemorrhage"
			why = "no hemorrhage bleed active and you need combo points"
			s.comboPoints = s.comboPoints + 1
			s.energy = s.energy - ns.e.Hemorrhage * eCostMult
			s.hemorrhageExpiration = s.now + 20
			s.playerIsStealthed = false

		elseif s.energy >= ns.e.Mutilate * eCostMult then

			spell = "Mutilate"
			why = "you need to build combo points"
			s.comboPoints = s.comboPoints + 2
			s.energy = s.energy - ns.e.Mutilate * eCostMult
			s.playerIsStealthed = false

		end
	end

	local vanishOk = s.vanishReadyTime <= s.now and s.isInGroup

	local vendettaOk = s.vendettaReadyTime <= s.now

	local tricksOfTheTradeOk = s.tricksOfTheTradeReadyTime <= s.now and s.isInGroup

	local kickOk = s.kickReadyTime <= s.now and s.interruptibleName ~= nil

	local cofsOk = s.cofsReadyTime <= s.now and s.removeableName ~= nil

	-- Update state to next intersting time.
	if spell ~= nil then
		s.now = max(s.now, oldNow + 1) -- global cool down
		s.globalCoolDownExpiration = s.now
	end
	s.now = max(s.now, oldNow + 0.2) -- Always move forward in time at least a little
	s.energy = s.energy + ns.combatEnergyRegenPerSec * (s.now - oldNow)

	return spell, vanishOk, vendettaOk, tricksOfTheTradeOk, kickOk, s.interruptibleName, s.longDurationDebuffs, s.removeableName, why
end

-- spell is a non-localised name.
function ns.saySpell(spell)
	if options.VoicePrompts then
		-- Try to play localised version, is present.
		local willPlay = PlaySoundFile("Interface\\AddOns\\Mutilate\\"..ns.n[spell]..".mp3", "Master")
		if not willPlay and ns.n[spell] ~= spell then
			-- Failing that, play English (if it is different)
			PlaySoundFile("Interface\\AddOns\\Mutilate\\"..spell..".mp3", "Master")
		end
	end
end

-- Blanks out all icon textures and forgets what we said last.
function ns.ClearUI()
	ns.textureRecommend:SetTexture(nil)
	ns.textureA1:SetTexture(nil)
	ns.textureA2:SetTexture(nil)
	ns.textureB1:SetTexture(nil)
	ns.textureB2:SetTexture(nil)
	ns.textureC1:SetTexture(nil)
	ns.textureD1:SetTexture(nil)
	ns.saidSpell = nil
	ns.saidKick = false
end

function ns.DecideSpells()

	local now = GetTime()
	if now - ns.lastUpdateTime < 0.05 then
		return
	end
	ns.lastUpdateTime = now
	
	local guid = UnitGUID("target")
	
	if  UnitName("target") == nil or UnitIsFriend("player", "target") or UnitHealth("target") == 0 then
		return -- ignore the dead and friendly
	end

	if guid == nil then
		ns.ClearUI()
		return
	end

	local s;
	--print("spec=", ns.spec)
	--if ns.spec == "Assasination" then
		s = ns.GetState(now)
	--else
		--s = ns.GetBalanceState(now)
	--end
	if s == nil then
		-- In period of confusion following starb of global cooldown.
		ns.textureRecommend:SetTexture(nil)
		return
	end

	-- print(ns.StateSummary(s))

	local globalCoolDownExpiration = s.globalCoolDownExpiration
	local spell, vanishOk, vendettaOk, tricksOfTheTradeOk, kickOk, interruptibleName, cofsOk, removeableName, why = ns.GetSpellFromState(s, "=")
	if spell == nil then
		ns.textureRecommend:SetTexture(nil)
		ns.saidSpell = nil
	else
		ns.textureRecommend:SetTexture(GetSpellTexture(ns.n[spell]))
		if ns.saidSpell ~= spell then
			ns.saySpell(spell)
			ns.saidSpell = spell
		end
	end
	
	if tricksOfTheTradeOk then
		ns.textureA1:SetTexture(GetSpellTexture(ns.n.Tricks_of_the_Trade))
	else
		ns.textureA1:SetTexture(nil)
	end

	if vanishOk then
		ns.textureB1:SetTexture(GetSpellTexture(ns.n.Vanish))
	else
		ns.textureB1:SetTexture(nil)
	end
	
	if vendettaOk then
		ns.textureB2:SetTexture(GetSpellTexture(ns.n.Vendetta))
	else
		ns.textureB2:SetTexture(nil)
	end

	if kickOk then
		ns.textureA2:SetTexture(GetSpellTexture(ns.n.Kick))
		ns.bottomText:SetText("|cffFFffFF"..interruptibleName)
		if not ns.saidKick then
			ns.saySpell("Kick")
			ns.saidKick = true
		end
	else
		ns.textureA2:SetTexture(nil)
		--ns.bottomText:SetText("") ####
		ns.saidKick = false
	end

	if cofsOk then
		ns.textureD1:SetTexture(GetSpellTexture(ns.n.Cloak_of_Shadows))
		ns.rightSideText:SetText("|cffFFffFF"..removeableName)
	else
		ns.textureD1:SetTexture(nil)
		ns.rightSideText:SetText("")
	end

	-- Try to predict next required spell using extrapolation of state.
	local i = 0
	local nextSpell = nil
	local whenNext = now
	while i < 20 do
		whenNext = s.now
		local nspell, vanishOk, vendettaOk, tricksOfTheTradeOk, kickOk, interruptibleName, cofsOk, removeableName, why = ns.GetSpellFromState(s, "+")
		nextSpell = nspell
		if nextSpell ~= nil then
			break
		end
		i = i + 1
	end
	if nextSpell == nil then 
		ns.textureC1:SetTexture(nil)
	else
		ns.textureC1:SetTexture(GetSpellTexture(ns.n[nextSpell]))
	end

end

function ns.CreateOptionFrame()
	local panel = CreateFrame("FRAME", "MutilateOptions");
	panel.name = "Mutilate";

	local y = -10
	local lineStep = 30

	local fstring1 = panel:CreateFontString("MutilateOptions_string1", "OVERLAY", "GameFontNormal")
	fstring1:SetText("Lock panel position")
	fstring1:SetPoint("TOPLEFT", 10, y)

	local checkbox1 = CreateFrame("CheckButton", "$parent_cb1", panel, "OptionsCheckButtonTemplate")
	checkbox1:SetWidth(18)
	checkbox1:SetHeight(18)
	checkbox1:SetScript("OnClick", function() ns.ToggleLocked() end)
	checkbox1:SetPoint("TOPRIGHT", -10, y)
	checkbox1:SetChecked(ns.GetLocked())
	y = y - lineStep
	
	--
	-- GUI Scale
	--
	local fstring5 = panel:CreateFontString("MutilateOptions_string5", "OVERLAY", "GameFontNormal")
	fstring5:SetText("GUI Scale")
	fstring5:SetPoint("TOPLEFT", 10, y)

	local slider5 = CreateFrame("Slider", "$parent_sl", panel, "OptionsSliderTemplate")
	slider5:SetMinMaxValues(.5, 1.5)
	slider5:SetValue(options.scale)
	slider5:SetValueStep(1.0 / 16)
	slider5:SetObeyStepOnDrag(true)
	slider5:SetScript("OnValueChanged", function(self) ns.SetScale(self:GetValue()); getglobal(self:GetName() .. "Text"):SetText(self:GetValue())  end)
	getglobal(slider5:GetName() .. "Low"):SetText("0.5")
	getglobal(slider5:GetName() .. "High"):SetText("1.5")
	getglobal(slider5:GetName() .. "Text"):SetText(options.scale)
	slider5:SetPoint("TOPRIGHT", -10, y)
	y = y - 2 * lineStep
	
	--
	-- Recuperate player health threshold
	--
	local fstring11 = panel:CreateFontString("MutilateOptions_string11", "OVERLAY", "GameFontNormal")
	fstring11:SetText("Suggest Recuperate when your health is below this %")
	fstring11:SetPoint("TOPLEFT", 10, y)

	local slider11 = CreateFrame("Slider", "$parent_s7", panel, "OptionsSliderTemplate")
	slider11:SetMinMaxValues(0, 100)
	slider11:SetValue(options.PlayerHealthThreshold)
	slider11:SetValueStep(1)
	slider11:SetObeyStepOnDrag(true)
	slider11:SetScript("OnValueChanged", function(self) ns.SetPlayerHealthThreshold(self:GetValue()); getglobal(self:GetName() .. "Text"):SetText(self:GetValue())  end)
	getglobal(slider11:GetName() .. "Low"):SetText("0")
	getglobal(slider11:GetName() .. "High"):SetText("100")
	getglobal(slider11:GetName() .. "Text"):SetText(options.PlayerHealthThreshold)
	slider11:SetPoint("TOPRIGHT", -10, y)
	y = y - lineStep

	-- Cloak of Shadows checkbox
	local fs = panel:CreateFontString("MutilateOptions_cofsCheck", "OVERLAY", "GameFontNormal")
	fs:SetText("Show Cloak of Shadows icon when you have debuffs")
	fs:SetPoint("TOPLEFT", 10, y)

	local cb = CreateFrame("CheckButton", "$parent_cb1", panel, "OptionsCheckButtonTemplate")
	cb:SetWidth(18)
	cb:SetHeight(18)
	cb:SetScript("OnClick", function() options.SuggestCloakOfShadows = not options.SuggestCloakOfShadows end)
	cb:SetPoint("TOPRIGHT", -10, y)
	cb:SetChecked(options.SuggestCloakOfShadows)
	y = y - lineStep
	
	local fs = panel:CreateFontString("MutilateOptions_cofsSlider", "OVERLAY", "GameFontNormal")
	fs:SetText("Only show Cloak of Shadows icon when a debuff has more duration than this")
	fs:SetPoint("TOPLEFT", 10, y)

	local slider = CreateFrame("Slider", "$parent_sCofs", panel, "OptionsSliderTemplate")
	slider:SetMinMaxValues(0, 60)
	slider:SetValue(options.MinDebuffDurationForCofs)
	slider:SetValueStep(1)
	slider:SetObeyStepOnDrag(true)
	slider:SetScript("OnValueChanged", function(self) options.MinDebuffDurationForCofs = self:GetValue(); getglobal(self:GetName() .. "Text"):SetText(self:GetValue())  end)
	getglobal(slider:GetName() .. "Low"):SetText("0")
	getglobal(slider:GetName() .. "High"):SetText("60")
	getglobal(slider:GetName() .. "Text"):SetText(options.MinDebuffDurationForCofs)
	slider:SetPoint("TOPRIGHT", -10, y)
	y = y - lineStep

	local cb = CreateFrame("CheckButton", "$parent_cb1", panel, "OptionsCheckButtonTemplate")
	cb:SetWidth(18)
	cb:SetHeight(18)
	cb:SetScript("OnClick", function() options.VoicePrompts = not options.VoicePrompts end)
	cb:SetPoint("TOPRIGHT", -10, y)
	cb:SetChecked(options.VoicePrompts)
	
	local fs = panel:CreateFontString("MutilateOptions_cofsSlider", "OVERLAY", "GameFontNormal")
	fs:SetText("Play voice prompts for main recommendation and kick")
	fs:SetPoint("TOPLEFT", 10, y)

	y = y - lineStep
	local fstring = panel:CreateFontString(nil, "OVERLAY", "GameFontNormal")
	fstring:SetText("|cffA0A0A0Logic based on:")
	fstring:SetPoint("TOPLEFT", 10, y)
	y = y - lineStep / 2

	local fstring = panel:CreateFontString(nil, "OVERLAY", "GameFontNormal")
	fstring:SetText("|cffA0A0A0http://www.icy-veins.com/wow/assassination-rogue-pve-dps-rotation-cooldowns-abilities")
	fstring:SetPoint("TOPLEFT", 10, y)
	y = y - lineStep

	local fstring = panel:CreateFontString(nil, "OVERLAY", "GameFontNormal")
	fstring:SetText("|cffA0A0A0This is an interim release for Legion pre patch. Plan to extend to other specs.")
	fstring:SetPoint("TOPLEFT", 10, y)
	y = y - lineStep / 2

	local fstring = panel:CreateFontString(nil, "OVERLAY", "GameFontNormal")
	fstring:SetText("|cffA0A0A0Requests, leave a note at: http://www.curse.com/addons/wow/mutilate")
	fstring:SetPoint("TOPLEFT", 10, y)
	y = y - lineStep

	InterfaceOptions_AddCategory(panel);
end

function ns.GetLocked()
	return options.locked
end

function ns.ToggleLocked()
	if options.locked then
		options.locked = false
		ns.displayFrame:SetScript("OnMouseDown", function(self) self:StartMoving() end)
		ns.displayFrame:SetScript("OnMouseUp", function(self) self:StopMovingOrSizing() end)
		ns.displayFrame:SetScript("OnDragStop", function(self) self:StopMovingOrSizing() end)
		ns.displayFrame:SetBackdropColor(0, 0, 0, .4)
		ns.displayFrame:EnableMouse(true)
	else
		options.locked = true
		ns.displayFrame:SetScript("OnMouseDown", nil)
		ns.displayFrame:SetScript("OnMouseUp", nil)
		ns.displayFrame:SetScript("OnDragStop", nil)
		ns.displayFrame:SetBackdropColor(0, 0, 0, 0)
		ns.displayFrame:EnableMouse(false)
	end
end


function ns.Options(msg, editBox)

	if (msg == 'show') then
		ns.displayFrame:Show() 
	elseif (msg == 'hide') then
		ns.displayFrame:Hide() 
	elseif (msg == '') then
		InterfaceOptionsFrame_OpenToCategory(getglobal("MutilateOptions"))
	end
	
end
	
function ns.SetScale(num)
	options.scale = num
	ns.displayFrame:SetScale(options.scale)
	ns.cooldownFrame:SetScale(options.scale)
end

function ns.SetPlayerHealthThreshold(num)
	options.PlayerHealthThreshold = num
end


function ns.shallowCopyOfTable(t)
  local t2 = {}
  for k, v in pairs(t) do
    t2[k] = v
  end
  return t2
end

